#!/usr/bin/env python
import os
import sys
from django.core.management import execute_from_command_line


if __name__ == '__main__':
    os.environ.setdefault(
        'DJANGO_SETTINGS_MODULE',
        'shuup_testutils.tests.workbench.settings',
    )
    execute_from_command_line(sys.argv)



from django.contrib.auth import get_user_model
from faker import Faker
from rest_framework.test import APIClient
from django.test.client import Client as DjangoTestClient


class AuthAdminClient(DjangoTestClient):
    user = None

    def init(self):
        """
        The `__init__` would be outside of the test transaction.
        """
        fake = Faker()
        user_ident = fake.user_name()
        self.user = get_user_model().objects.create(
            username=user_ident,
            email=user_ident + '@localhost',
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            password=user_ident,
            is_superuser=True,
            is_staff=True,
        )
        super().force_login(self.user)


class AuthUserClient(DjangoTestClient):
    user = None

    def init(self):
        """
        The `__init__` would be outside of the test transaction.
        """
        fake = Faker()
        user_ident = fake.user_name()
        self.user = get_user_model().objects.create(
            username=user_ident,
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            email=user_ident + '@localhost',
            password=user_ident,
        )
        super().force_login(self.user)


class ApiAuthAdminClient(APIClient, AuthAdminClient):
    pass



from typing import Callable

from shuup_testutils.generators.unique import UniqueGen
from shuup_testutils.generators.unique import GeneratedValue
from shuup_testutils.generators.unique import UniqueGenError
from shuup_testutils.cases import SnakeTestCase


class UniqueGeneratorTest(SnakeTestCase):
    unique_gen = None

    def set_up(self):
        super().set_up()
        self.unique_gen = UniqueGen()

    def test_unique_word_generator_limit(self):
        with self.assert_raises(UniqueGenError):
            for _ in range(0, 500):
                self.unique_gen.word()

    def test_unique_word_uniqueness(self):
        self._test_uniqueness(generator_fn=self.unique_gen.word)

    def test_unique_int_uniqueness(self):
        self._test_uniqueness(generator_fn=self.unique_gen.integer)

    def _test_uniqueness(
        self,
        generator_fn: Callable[[], GeneratedValue],
        values_amount: int = 150,
    ):
        values_generated = []
        for _ in range(0, values_amount):
            value_generated = generator_fn()
            self.assert_true(value_generated not in values_generated)
            values_generated.append(value_generated)



from typing import Union, Callable, List

from faker import Faker


GeneratedValue = Union[str, int]


class UniqueGen:
    _fake = None

    _generated_unique_words = None
    _generated_unique_ints = None

    def __init__(self):
        self._fake = Faker()
        self._generated_unique_words = []
        self._generated_unique_ints = []

    def word(self) -> str:
        word = self._unique_value(
            generator_fn=self._fake.word,
            generated_values_list=self._generated_unique_words,
        )
        return word

    def integer(self) -> int:
        integer = self._unique_value(
            generator_fn=self._fake.pyint,
            generated_values_list=self._generated_unique_ints,
        )
        return integer

    def _unique_value(
        self,
        generator_fn: Callable[[], GeneratedValue],
        generated_values_list: List[GeneratedValue],
        attempts_limit: int = 400,
    ) -> GeneratedValue:
        attempts_count = 0
        while attempts_count < attempts_limit:
            attempts_count += 1

            generated_value = generator_fn()
            is_generated_unique = generated_value not in generated_values_list
            if is_generated_unique:
                generated_values_list.append(generated_value)
                return generated_value
        else:
            raise UniqueGenError(
                "Unable to generate any more unique words. Usually if Faker "
                "didn't generate a unique word after ~200 attempt - he won't."
            )


class UniqueGenError(Exception):
    pass



from shuup_testutils.cases.snake import SnakeTestCase
from shuup_testutils.cases.clients import AuthAdminClient
from shuup_testutils.cases.clients import AuthUserClient
from shuup_testutils.cases.clients import ApiAuthAdminClient


class ApiAuthAdminTestCase(SnakeTestCase):
    client_class = ApiAuthAdminClient

    @classmethod
    def set_up_class(cls):
        cls.init_shuup()

    def set_up(self):
        self.client.init()


class AuthAdminTestCase(SnakeTestCase):
    client_class = AuthAdminClient

    @classmethod
    def set_up_class(cls):
        cls.init_shuup()

    def set_up(self):
        self.client.init()


class AuthUserTestCase(SnakeTestCase):
    client_class = AuthUserClient

    @classmethod
    def set_up_class(cls):
        cls.init_shuup()

    def set_up(self):
        self.client.init()



from django.http import HttpRequest
from django.test import RequestFactory
from shuup.testing.factories import get_default_shop
from shuup.testing.factories import create_random_person
from shuup.testing.utils import apply_request_middleware


def gen_shuup_request(factory: RequestFactory) -> HttpRequest:
    request = factory.get('/')
    request.shop = get_default_shop()
    request.customer = create_random_person()
    apply_request_middleware(request)
    return request



import re

from django.core.urlresolvers import reverse
from shuup.core.models import Product
from shuup_testutils.generators.shuup.admin_forms import ShuupProductAdminFormsGen
from shuup_testutils.generators.shuup.models import ShuupModelsGen
from shuup_testutils.cases import AuthAdminTestCase


class ShuupFormGenTest(AuthAdminTestCase):
    @classmethod
    def set_up_class(cls):
        super().set_up_class()
        cls.gen = ShuupModelsGen()

    def test_barcode_update(self):
        product_barcode_new = '123123123'
        product = self.gen.product()
        form_gen = ShuupProductAdminFormsGen(product_pk=product.pk)
        form_gen.set_barcode_field(product_barcode_new)
        form = form_gen.get_form()
        response = self.client.post(
            reverse('shuup_admin:shop_product.edit', kwargs={'pk': product.pk}),
            data=form,
            follow=True,
        )
        product_updated = Product.objects.get(pk=product.pk)
        self.assert_contains(response, product_barcode_new)
        self.assert_equal(product_barcode_new, product_updated.barcode)

    def test_name_update(self):
        product_name_new = 'product_name_new'
        product = self.gen.product()
        form_gen = ShuupProductAdminFormsGen(product_pk=product.pk)
        form_gen.set_name_field(product_name_new)
        form = form_gen.get_form()
        response = self.client.post(
            reverse('shuup_admin:shop_product.edit', kwargs={'pk': product.pk}),
            data=form,
            follow=True,
        )
        product_updated = Product.objects.get(pk=product.pk)
        self.assert_contains(response, product_name_new)
        self.assert_not_contains(response, 'This field is required')
        self.assert_equal(product_name_new, product_updated.name)

    def test_product_new_post(self):
        form_gen = ShuupProductAdminFormsGen()
        form_gen.set_name_field('some product name')
        form_gen.set_price_field(42)
        response = self.client.post(
            path=reverse('shuup_admin:shop_product.new'),
            data=form_gen.get_form(),
            follow=True,
        )
        product_created_url = response.wsgi_request.path
        match = re.match('/sa/products/(?P<id>[0-9]+)/', product_created_url)
        is_product_created = match is not None
        self.assert_true(is_product_created)



from django.http import HttpResponse
from django.test import TestCase
from shuup import configuration
from shuup.core.models import Shop
from shuup.xtheme import set_current_theme
from shuup.testing.factories import get_default_shop

from shuup_testutils.settings_base import SHUUP_DEFAULT_THEME


class SnakeTestCase(TestCase):
    @classmethod
    def set_up_class(cls):
        pass

    def set_up(self):
        pass

    @classmethod
    def init_shuup(cls):
        get_default_shop()
        set_current_theme(SHUUP_DEFAULT_THEME)
        configuration.set(Shop.objects.first(), 'setup_wizard_complete', True)

    def assert_true(self, expression, fail_message: str = None):
        return self.assertTrue(expression, msg=fail_message)

    def assert_false(self, expression, fail_message: str = None):
        return self.assertFalse(expression, msg=fail_message)

    def assert_equal(self, expected, actual, fail_message: str = None):
        return self.assertEqual(expected, actual, msg=fail_message)

    def assert_not_equal(self, expected, actual, fail_message: str = None):
        return self.assertNotEqual(expected, actual, fail_message)

    def assert_raises(self, exception_cls, callable_obj=None, *args, **kwargs):
        return self.assertRaises(exception_cls, callable_obj, *args, **kwargs)

    def assert_num_queries(self, *args, **kwargs):
        return self.assertNumQueries(*args, **kwargs)

    def assert_redirects(
        self,
        response: HttpResponse,
        expected_url: str,
        status_code: int = 302,
        target_status_code: int = 200,
        msg_prefix: str = '',
        fetch_redirect_response: bool = True,
    ):
        return self.assertRedirects(
            response, expected_url, status_code, target_status_code, None,
            msg_prefix, fetch_redirect_response,
        )

    def assert_contains(
        self,
        response: HttpResponse,
        text,
        count=None,
        status_code: int = 200,
        msg_prefix='',
        html=False,
    ):
        return self.assertContains(
            response,
            text,
            count=count,
            status_code=status_code,
            msg_prefix=msg_prefix,
            html=html,
        )

    def assert_not_contains(
        self,
        response,
        text,
        status_code=200,
        msg_prefix='',
        html=False,
    ):
        return self.assertNotContains(
            response,
            text,
            status_code=status_code,
            msg_prefix=msg_prefix,
            html=html,
        )

    def assert_in(self, member, container, msg: str = None):
        self.assertIn(member, container, msg)

    def assert_not_in(self, member, container, msg: str = None):
        self.assertNotIn(member, container, msg)

    def setUp(self):
        self.set_up()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.set_up_class()



import setuptools


def main():
    setuptools.setup(
        name='shuup-testutils',
        version='0.7.0.1',
        description='Some personal testutils for Shuup E-commerce Platform',
        url='https://gitlab.com/nilit/shuup-testutils',
        license='MIT',
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Framework :: Django :: 1.9',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Topic :: Software Development',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Software Development :: Testing',
            'Topic :: Internet :: WWW/HTTP',
            'Topic :: Office/Business',
        ],
        keywords='shuup',
        packages=setuptools.find_packages(),
        install_requires=[
            'shuup >= 1.1, < 2',
            'shuup-utils >= 0.4, < 0.5',
            'beautifulsoup4 >= 4.5.3, < 4.6',
        ],
        extras_require={
            'dev': [
                'wheel >= 0.29.0',
                'twine >= 1.8.1, < 2',
            ],
        },
        include_package_data=True,
    )


if __name__ == '__main__':
    main()



from decimal import Decimal
from typing import List, Union

from django.contrib.auth import get_user_model
from django.db.models import Model
from shuup.core.models import Shop
from shuup.core.models import Supplier
from shuup.testing.factories import get_default_supplier
from shuup.testing.factories import get_default_sales_unit
from shuup.testing.factories import get_default_product_type
from shuup.testing.factories import get_default_tax_class
from shuup.testing.factories import get_default_category
from shuup.testing.factories import get_default_shop
from shuup.testing.factories import get_random_filer_image
from shuup.core.models import ProductMedia
from shuup.core.models import CategoryStatus
from shuup.core.models import ProductMediaKind
from shuup.core.models import StockBehavior
from shuup.core.models import ShopProduct
from shuup.core.models import Product
from shuup.core.models import Category
from shuup.core.models import ShopProductVisibility
from faker import Faker

from shuup_testutils.generators import UniqueGen


class ShuupModelsGen:
    _fake = None

    def __init__(self):
        self._fake = Faker()
        self._unique_gen = UniqueGen()

    def user(self, user_model: Model = None, is_superuser: bool = False) -> Model:
        user_model = user_model or get_user_model()
        # TODO: may Fake fail to create a unique username/email?
        user_data = dict(
            username=self._fake.user_name(),
            email=self._fake.email(),
            password=self._fake.password(),
            first_name=self._fake.first_name(),
            last_name=self._fake.last_name(),
            last_login=self._fake.date_time(),
            date_joined=self._fake.date_time(),
        )
        if is_superuser:
            return user_model.objects.create_superuser(**user_data)
        else:
            return user_model.objects.create(**user_data)

    # noinspection PyDefaultArgument
    def product(
        self,
        name: str = None,
        shop: Shop = None,
        price: Union[int, float, Decimal, str, None] = Decimal(500),
        primary_category: Category = None,
        categories: List[Category] = [],
        product_model = Product,
        supplier: Supplier = None,
    ) -> Product:
        product = product_model(
            type=get_default_product_type(),
            tax_class=get_default_tax_class(),
            sku=self._unique_gen.word(),
            name=name or self._unique_gen.word().title(),
            width=0, height=0, depth=0, net_weight=0, gross_weight=0,
            sales_unit=get_default_sales_unit(),
            stock_behavior=StockBehavior.UNSTOCKED,
        )
        product.full_clean()
        product.save()
        # `full_clean` won't allow a category to be passed as a kwarg above
        product.category = primary_category or get_default_category()
        product.save()

        self._add_image_to_product(product)
        
        shop_instance = shop or get_default_shop()
        shop_product = ShopProduct.objects.create(
            product=product,
            shop=shop_instance,
            default_price=shop_instance.create_price(price) if price else None,
            primary_category=primary_category or get_default_category(),
            visibility=ShopProductVisibility.ALWAYS_VISIBLE,
        )
        shop_product.categories = categories
        shop_product.suppliers.add(supplier or get_default_supplier())
        shop_product.save()

        return product

    def category(self, parent_category: Category = None) -> Category:
        category = Category.objects.create(
            parent=parent_category,
            identifier=self._unique_gen.word(),
            name=self._unique_gen.word().title(),
            slug=self._unique_gen.word(),
            status=CategoryStatus.VISIBLE,
        )
        return category

    def _add_image_to_product(self, product: Product):
        primary_image = ProductMedia.objects.create(
            product=product,
            kind=ProductMediaKind.IMAGE,
            file=get_random_filer_image(),
            enabled=True,
            public=True,
        )
        for shop in Shop.objects.all():
            primary_image.shops.add(shop)
        primary_image.save()
        product.primary_image = primary_image
        product.save()
